unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Smiley, StdCtrls;

const
  NrOfSmileys = 8;
  NrOfMsgs = 16;
  Msg : array [1..NrOfMsgs] of String =
          ('Smile''n''Laugh, Inc. proudly presents...',
           'The Smiley Game',
           'Written one rainy night in June, 1995',
           'by me - Anders Ohlsson',
           'ao@it.kth.se - http://www.it.kth.se/~ao',
           'I''d like to take the opportunity to blame',
           'Lt. John N. Hodges (Nick)',
           'at the Naval Postgraduate School',
           'for all this, since he''s the one',
           'who wrote the infamous TSmiley component',
           'Hi Nick! ;-)',
           '',
           '...and remember...',
           '',
           'you may have to cheat...',
           '');

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    SmileyPanel: TPanel;
    Panel4: TPanel;
    MrSmiley: TImage;
    Memo1: TMemo;
    Panel5: TPanel;
    GameOverMemo: TMemo;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure NewGame(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure MrSmileyMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MrSmileyMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MrSmileyMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    Loading,
    DragOn     : Boolean;
    SmileyFace : array [1..NrOfSmileys,1..NrOfSmileys] of TSmiley;
    procedure CheckWin;
    procedure SmileyClick(Sender : TObject);
    procedure SmileyDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SmileyDragDrop(Sender, Source: TObject; X, Y: Integer);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Randomize;
  Loading := True;
end;

procedure TMainForm.FormActivate(Sender: TObject);
var
  i, j : Integer;
begin
  if Loading then begin
    Loading := False;

    for i:=1 to NrOfSmileys do
      for j:=1 to NrOfSmileys do begin
        SmileyFace[i,j] := TSmiley.Create(MainForm);

        with SmileyFace[i,j] do begin
          Parent := SmileyPanel;
          Top := 8+(i-1)*32;
          Left := 8+(j-1)*32;
          Tag := i-1+NrOfSmileys*(j-1);
          OnClick := SmileyClick;
          OnDragOver := SmileyDragOver;
          OnDragDrop := SmileyDragDrop;
        end;
      end;

    NewGame(nil);
  end;
end;

procedure TMainForm.CheckWin;
var
  GameOver : Boolean;
  i, j     : Integer;
begin
  GameOver := True;
  for i:=1 to NrOfSmileys do
    for j:=1 to NrOfSmileys do
      if SmileyFace[i,j].Mood = smOoh then
        GameOver := False;

  if GameOver then
    GameOverMemo.Visible := True;
end;

procedure TMainForm.SmileyClick(Sender: TObject);
var
  x, y, cx, cy, i, j : Integer;
begin
  x := ((Sender as TSmiley).Tag mod NrOfSmileys)+1;
  y := ((Sender as TSmiley).Tag div NrOfSmileys)+1;

  for i:=-1 to 1 do
    for j:=-1 to 1 do
      if (i+j)*(i+j) = 1 then begin
        cx := x+i;
        if cx < 1 then
          cx := NrOfSmileys;
        if cx > NrOfSmileys then
          cx := 1;

        cy := y+j;
        if cy < 1 then
          cy := NrOfSmileys;
        if cy > NrOfSmileys then
          cy := 1;

        if SmileyFace[cx,cy].Mood = smHappy then
          SmileyFace[cx,cy].Mood := smOoh
        else
          SmileyFace[cx,cy].Mood := smHappy;
      end;

  CheckWin;
end;

procedure TMainForm.SmileyDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := Source = MrSmiley;
end;

procedure TMainForm.SmileyDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  (Sender as TSmiley).Mood := smHappy;

  CheckWin;
end;

procedure TMainForm.NewGame(Sender: TObject);
var
  i, j : Integer;
begin
  GameOverMemo.Visible := False;

  for i:=1 to NrOfSmileys do
    for j:=1 to NrOfSmileys do
        with SmileyFace[i,j] do
          if Random(3) = 1 then
            Mood := smOoh
          else
            Mood := smHappy;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
const
  i : Integer = 1;
begin
  Caption := Msg[i];

  Inc(i);
  if i > NrOfMsgs then
    i := 1;
end;

procedure TMainForm.MrSmileyMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    DragOn := True;
end;

procedure TMainForm.MrSmileyMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if DragOn then
    MrSmiley.BeginDrag(False);
end;

procedure TMainForm.MrSmileyMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    DragOn := False;
end;

end.
